package org.jvnet.hifaces.application.navigation.impl;

import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.ServletRequest;

public class NavigationPhaseListener implements PhaseListener {

	private static final long serialVersionUID = 1L;

	public PhaseId getPhaseId() {
		return PhaseId.RESTORE_VIEW;
	}

	public void beforePhase(PhaseEvent event) {
		final FacesContext facesContext = event.getFacesContext();

		final ServletRequest request = (ServletRequest) facesContext
				.getExternalContext().getRequest();

		final Long previousStateId = getLongRequestAttribute(facesContext,
				DefaultNavigator.CURRENT_STATE_ID_REQUEST_PARAMETER_NAME);

		final State previousState = State.getState(facesContext,
				previousStateId);

		final Long currentStateId = getLongRequestAttribute(facesContext,
				DefaultNavigator.NEXT_STATE_ID_REQUEST_PARAMETER_NAME);

		final State draftCurrentState = State.getState(facesContext,
				currentStateId);

		final State preCurrentState;

		if (previousState == null && draftCurrentState == null) {
			preCurrentState = null;

		} else if (previousState != null && draftCurrentState == null) {
			preCurrentState = previousState;
		} else if (previousState == null && draftCurrentState != null) {
			preCurrentState = draftCurrentState;
		} else {
			preCurrentState = draftCurrentState;
		}

		final State currentState = preCurrentState;

		request.setAttribute(
				DefaultNavigator.CURRENT_STATE_REQUEST_ATTRIBUTE_NAME,
				currentState);
	}

	public void afterPhase(PhaseEvent event) {
		final FacesContext facesContext = event.getFacesContext();
		final ServletRequest request = (ServletRequest) facesContext
				.getExternalContext().getRequest();

		final State draftCurrentState = (State) request
				.getAttribute(DefaultNavigator.CURRENT_STATE_REQUEST_ATTRIBUTE_NAME);

		final UIViewRoot viewRoot = facesContext.getViewRoot();
		if (draftCurrentState == null && viewRoot != null) {
			final String viewId = viewRoot.getViewId();
			final State currentState;
			currentState = State.createState(facesContext, viewId);
			request.setAttribute(
					DefaultNavigator.CURRENT_STATE_REQUEST_ATTRIBUTE_NAME,
					currentState);
		}
	}

	private static Long getLongRequestAttribute(FacesContext facesContext,
			String name) {
		final String parameter = (String) facesContext.getExternalContext()
				.getRequestParameterMap().get(name);
		if (parameter != null) {

			try {
				final Long value = Long.parseLong(parameter);
				return value;
			} catch (NumberFormatException nfe) {
				return null;
			}
		} else {
			return null;
		}
	}

}
