package org.jvnet.hifaces.components.tabbedpane;

import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlPanelGroup;
import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.FacesEvent;

import org.apache.myfaces.custom.tabbedpane.TabChangeEvent;
import org.apache.myfaces.custom.tabbedpane.TabChangeListener;

public class HtmlPanelTabbedPane extends HtmlPanelGroup {

	private String tableStyleClass = Constants.DEFAULT_TABLE_STYLE_CLASS;

	private String labelsRowStyleClass = Constants.DEFAULT_LABELS_ROW_STYLE_CLASS;

	private String contentsRowStyleClass = Constants.DEFAULT_CONTENTS_ROW_STYLE_CLASS;

	private String preColumnStyleClass = Constants.DEFAULT_PRE_COLUMN_STYLE_CLASS;

	private String activeColumnStyleClass = Constants.DEFAULT_ACTIVE_COLUMN_STYLE_CLASS;

	private String midColumnStyleClass = Constants.DEFAULT_MID_COLUMN_STYLE_CLASS;

	private String inactiveColumnStyleClass = Constants.DEFAULT_INACTIVE_COLUMN_STYLE_CLASS;

	private String postColumnStyleClass = Constants.DEFAULT_POST_COLUMN_STYLE_CLASS;

	private String contentColumnStyleClass = Constants.DEFAULT_CONTENT_COLUMN_STYLE_CLASS;

	private MethodBinding _tabChangeListener = null;

	public void addTabChangeListener(TabChangeListener listener) {
		addFacesListener(listener);
	}

	public void removeTabChangeListener(TabChangeListener listener) {
		removeFacesListener(listener);
	}

	public MethodBinding getTabChangeListener() {
		return _tabChangeListener;
	}

	public void setTabChangeListener(MethodBinding tabChangeListener) {
		_tabChangeListener = tabChangeListener;
	}

	public void broadcast(FacesEvent event) throws AbortProcessingException {
		if (event instanceof TabChangeEvent) {
			TabChangeEvent tabChangeEvent = (TabChangeEvent) event;
			if (tabChangeEvent.getComponent() == this) {
				setSelectedIndex(tabChangeEvent.getNewTabIndex());
				getFacesContext().renderResponse();
			}
		}
		super.broadcast(event);

		MethodBinding tabChangeListenerBinding = getTabChangeListener();
		if (tabChangeListenerBinding != null) {
			try {
				tabChangeListenerBinding.invoke(getFacesContext(),
						new Object[] { event });
			} catch (EvaluationException e) {
				Throwable cause = e.getCause();
				if (cause != null && cause instanceof AbortProcessingException) {
					throw (AbortProcessingException) cause;
				} else {
					throw e;
				}
			}
		}
	}

	public static final String COMPONENT_TYPE = "org.jvnet.hifaces.HtmlPanelTabbedPane";

	public static final String COMPONENT_FAMILY = "javax.faces.Panel";

	public static final String DEFAULT_RENDERER_TYPE = "org.jvnet.hifaces.TabbedPane";

	private static final int DEFAULT_SELECTEDINDEX = 0;

	private Integer _selectedIndex = null;

	public HtmlPanelTabbedPane() {
		setRendererType(DEFAULT_RENDERER_TYPE);
	}

	public String getFamily() {
		return COMPONENT_FAMILY;
	}

	public void setSelectedIndex(int selectedIndex) {
		_selectedIndex = new Integer(selectedIndex);
		for (int index = 0; index < getChildCount(); index++) {
			final UIComponent child = (UIComponent) getChildren().get(index);
			final UIComponent childContent = child.getFacet("content");
			if (childContent != null)
				childContent.setRendered(index == selectedIndex);
		}
	}

	public int getSelectedIndex() {
		if (_selectedIndex != null) {
			return _selectedIndex.intValue();
		}
		final ValueBinding vb = getValueBinding("selectedIndex");
		final Number v = vb != null ? (Number) vb.getValue(getFacesContext())
				: null;
		return v != null ? v.intValue() : DEFAULT_SELECTEDINDEX;
	}

	public Object saveState(FacesContext context) {
		final Object values[] = new Object[3];
		values[0] = super.saveState(context);
		values[1] = _selectedIndex;
		values[2] = saveAttachedState(context, _tabChangeListener);
		return ((Object) (values));
	}

	public void restoreState(FacesContext context, Object state) {
		final Object values[] = (Object[]) state;
		super.restoreState(context, values[0]);
		_selectedIndex = (Integer) values[1];
		_tabChangeListener = (MethodBinding) restoreAttachedState(context,
				values[2]);
	}

	public String getActiveColumnStyleClass() {
		return activeColumnStyleClass;
	}

	public String getContentColumnStyleClass() {
		return contentColumnStyleClass;
	}

	public String getContentsRowStyleClass() {
		return contentsRowStyleClass;
	}

	public String getInactiveColumnStyleClass() {
		return inactiveColumnStyleClass;
	}

	public String getLabelsRowStyleClass() {
		return labelsRowStyleClass;
	}

	public String getMidColumnStyleClass() {
		return midColumnStyleClass;
	}

	public String getPostColumnStyleClass() {
		return postColumnStyleClass;
	}

	public String getPreColumnStyleClass() {
		return preColumnStyleClass;
	}

	public String getTableStyleClass() {
		return tableStyleClass;
	}
}
