package org.jvnet.hifaces.components.templatedpanel.impl;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.faces.context.FacesContext;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.core.io.Resource;

public class SpringTemplateNameResolver implements TemplateNameResolver,
		ApplicationContextAware, InitializingBean {

	private ApplicationContext applicationContext;

	public void setApplicationContext(ApplicationContext applicationContext)
			throws BeansException {
		this.applicationContext = applicationContext;
	}

	public ApplicationContext getApplicationContext() {
		return applicationContext;
	}

	private List<String> templateLocations = new ArrayList<String>();

	public List<String> getTemplateLocations() {
		return templateLocations;
	}

	public void setTemplateLocations(List<String> templateLocations) {
		this.templateLocations = templateLocations;
	}

	private List<Resource> templateResources = new ArrayList<Resource>();

	public List<Resource> getTemplateResources() {
		return templateResources;
	}

	public void setTemplateResources(List<Resource> templateResources) {
		this.templateResources = templateResources;
	}

	private boolean classPathTemplateResources = true;

	public boolean isClassPathTemplateResources() {
		return classPathTemplateResources;
	}

	public void setClassPathTemplateResources(boolean classPathTemplateResources) {
		this.classPathTemplateResources = classPathTemplateResources;
	}

	public void afterPropertiesSet() throws Exception {
		for (final String templateLocation : getTemplateLocations()) {
			final Resource[] templateResources = getApplicationContext()
					.getResources(templateLocation);
			for (final Resource templateResource : templateResources) {
				this.templateResources.add(templateResource);
			}
		}
	}

	public TemplateResource resolve(FacesContext contex, String templateName)
			throws IOException {

		for (final Resource templateResource : getTemplateResources()) {
			final Resource draftResource = templateResource
					.createRelative(templateName);
			if (draftResource.exists()) {
				return createTemplateResource(draftResource);
			}
		}

		if (isClassPathTemplateResources()) {
			final Resource draftResource = applicationContext
					.getResource("classpath:" + templateName);
			if (draftResource.exists()) {
				return createTemplateResource(draftResource);
			}
		}

		{
			final Resource draftResource = getApplicationContext().getResource(
					templateName);
			if (draftResource.exists()) {
				return createTemplateResource(draftResource);
			}
		}

		// If it's neither file nor url, throw an exception
		throw new IOException("Template name [" + templateName
				+ "] could not be resolved.");
	}

	public TemplateResource createTemplateResource(final Resource resource) {
		// First see if it's a file
		try {
			final File file = resource.getFile();
			return new FileTemplateResource(file);
		} catch (Exception ex) {
			// Nothing to do
		}

		// Second see if it's an URL
		try {
			final URL url = resource.getURL();
			return new URLTemplateResource(url);
		} catch (Exception ex) {
			// Nothing to do
		}

		throw new IllegalArgumentException("Resource is neither file nor URL.");
	}

}
