package org.jvnet.hifaces.function;

import java.io.Serializable;
import java.lang.reflect.Method;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodNotFoundException;
import javax.faces.el.ValueBinding;

import org.apache.commons.beanutils.MethodUtils;
import org.jvnet.hifaces.function.impl.Method0;
import org.jvnet.hifaces.function.impl.Method1;
import org.jvnet.hifaces.function.impl.Method2;
import org.jvnet.hifaces.function.util.FunctionUtils;

/**
 * Function of no parameters.
 * 
 * @author valikov
 * 
 * @param <R>
 *            Return type.
 */
public abstract class Function0<R> extends Function<R> implements Serializable {

	public Function0(final Class<? extends R> type) {
		super(type);
	}

	@Override
	public Object invoke(FacesContext facescontext, Object[] aobj)
			throws EvaluationException, MethodNotFoundException {
		assert aobj.length == 0;
		try {
			return invoke(facescontext);
		} catch (Exception ex) {
			throw new EvaluationException(ex);
		}
	}

	public R invoke(FacesContext facescontext) throws Exception {
		return call();
	}

	public R call() throws Exception {
		return null;
	}

	public ValueBinding asValueBinding() {
		return asProperty();
	}

	public Property0<R> asProperty() {
		return new Property0<R>(this);
	}

	public <T> Function0<T> method(final String methodName) {
		final Method method = MethodUtils.getAccessibleMethod(getType(),
				methodName, (Class[]) null);
		if (method == null)
			throw new IllegalArgumentException("Method [" + methodName
					+ "]does not exist.");
		final Method0<T, R> m = new Method0<T, R>(method);
		return m.compose(this);
	}

	public <T, P> Function1<T, P> method(final String methodName,
			Class<? extends P> arg0) {

		final Method method = MethodUtils.getAccessibleMethod(getType(),
				methodName, new Class[] { arg0 });
		if (method == null)
			throw new IllegalArgumentException("Method [" + methodName
					+ "]does not exist.");

		final Method1<T, R, P> m = new Method1<T, R, P>(method);
		return m.compose0(this);
	}

	public <T> Property0<T> property(final String name) {

		final Method getter = FunctionUtils.getter(getType(), name);

		if (getter == null) {

			throw new IllegalArgumentException(
					"Invalid property name, getter could not be found.");
		} else {
			final Function0<T> get = new Method0<T, R>(getter).compose(this);
			final Method setter = FunctionUtils.setter(getType(), name, getter
					.getReturnType());
			if (setter == null) {
				return new Property0<T>(get);
			} else {
				final Function1<Void, T> set = new Method1<Void, R, T>(setter)
						.compose0(this);
				return new Property0<T>(get, set);
			}

		}
	}

	public <T, P> Property1<T, P> property(final String name,
			Class<? extends P> arg) {

		final Method getter = FunctionUtils.getter(getType(), name,
				new Class[] { arg });

		if (getter == null) {

			throw new IllegalArgumentException(
					"Invalid property name, getter could not be found.");
		} else {
			final Function1<T, P> get = new Method1<T, R, P>(getter)
					.compose0(this);
			final Method setter = FunctionUtils.setter(getType(), name,
					new Class[] { arg }, getter.getReturnType());
			if (setter == null) {
				return new Property1<T, P>(get);
			} else {
				final Function2<Void, P, T> set = new Method2<Void, R, P, T>(
						setter).compose0(this);
				return new Property1<T, P>(get, set);
			}

		}
	}

}
