package org.jvnet.hifaces.function;

import java.io.Serializable;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodNotFoundException;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ValueChangeEvent;

public abstract class Function1<R, P> extends Function<R> implements
		Serializable {

	public Function1(Class<? extends R> type) {
		super(type);
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object invoke(FacesContext facesContext, Object[] aobj)
			throws EvaluationException, MethodNotFoundException {
		assert aobj.length == 1;
		try {
			final P arg = (P) aobj[0];
			return invoke(facesContext, arg);
		} catch (Exception ex) {
			throw new EvaluationException(ex);
		}
	}

	public R invoke(FacesContext facesContext, P arg) throws Exception {
		return call(arg);
	}

	public R call(P arg) throws Exception {
		return null;
	}

	public Function0<R> bind(final P arg) {
		return new Bound0<R, P>(this, arg);
	}

	public Function0<R> compose(final Function0<P> function0) {
		return new Composed0<R, P>(this, function0);
	}

	public static class Bound0<R, P> extends Function0<R> {
		private static final long serialVersionUID = 1L;

		private final Function1<R, P> function1;

		private final P arg;

		public Bound0(Function1<R, P> function1, P arg) {
			super(function1.getType());
			this.function1 = function1;
			this.arg = arg;
		}

		@Override
		public R invoke(FacesContext facescontext) throws Exception {
			return function1.invoke(facescontext, arg);
		}

		@Override
		public R call() throws Exception {
			return function1.call(arg);
		}
	}

	public static class Composed0<R, P> extends Function0<R> {
		private static final long serialVersionUID = 1L;

		private final Function1<R, P> function1;

		private final Function0<P> function0;

		public Composed0(Function1<R, P> function1, Function0<P> function0) {
			super(function1.getType());
			this.function1 = function1;
			this.function0 = function0;
		}

		@Override
		public R invoke(FacesContext context) throws Exception {
			return function1.invoke(context, function0.invoke(context));
		}

		@Override
		public R call() throws Exception {
			return function1.call(function0.call());
		}
	}
}
