package org.jvnet.hifaces.util;

import javax.faces.component.UICommand;
import javax.faces.component.html.HtmlCommandButton;
import javax.faces.component.html.HtmlCommandLink;
import javax.faces.component.html.HtmlGraphicImage;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;

import org.jvnet.hifaces.renderkit.html.ext.HtmlButtonRenderer;
import org.jvnet.hifaces.text.Text;

public class CommandUtils {

	public static HtmlCommandLink createCommandLink(final ValueBinding value,
			final MethodBinding action) {
		final HtmlCommandLink commandLink = createCommandLink(action);
		commandLink.setValueBinding("value", value);
		return commandLink;
	}

	public static HtmlCommandLink createCommandLink(final MethodBinding action) {
		final HtmlCommandLink commandLink = createCommandLink();
		if (action != null) {
			commandLink.setAction(action);
		}
		return commandLink;
	}

	public static HtmlCommandLink createCommandLink() {
		final HtmlCommandLink commandLink = new HtmlCommandLink();
		IdUtils.assignId(commandLink);
		return commandLink;
	}

	public static HtmlCommandLink createImageCommandLink(final String url,
			final MethodBinding action, final ValueBinding title) {
		final HtmlCommandLink link = createCommandLink();
		final HtmlGraphicImage image = ImageUtils
				.createGraphicImage(url, title);
		link.getChildren().add(image);
		if (action != null) {
			link.setAction(action);
		}
		return link;
	}

	public static HtmlCommandLink createImageCommandLink(final String url,
			final MethodBinding action, final String title) {
		return createImageCommandLink(url, action, new Text(title));
	}

	public static HtmlCommandButton createCommandButton() {
		final HtmlCommandButton commandButton = new HtmlCommandButton();
		IdUtils.assignId(commandButton);
		commandButton.setRendererType(HtmlButtonRenderer.RENDERER_TYPE);
		return commandButton;
	}

	public static HtmlCommandButton createCommandButton(MethodBinding action) {
		final HtmlCommandButton commandButton = createCommandButton();
		if (action != null) {
			commandButton.setAction(action);
		}
		return commandButton;
	}

	public static HtmlCommandButton createCommandButton(ValueBinding value,
			MethodBinding action) {
		final HtmlCommandButton commandButton = createCommandButton(action);
		commandButton.setValueBinding("value", value);
		return commandButton;
	}

	public static HtmlCommandButton createCommandButton(String messageId,
			MethodBinding action) {
		final HtmlCommandButton commandButton = createCommandButton(action);
		commandButton.setValueBinding("value", new Text(messageId));
		return commandButton;
	}

	
	public static UICommand createCommand() {
		final UICommand command = new UICommand();
		IdUtils.assignId(command);
		return command;
	}

	public static UICommand createCommand(MethodBinding action) {
		final UICommand command = createCommand();
		command.setAction(action);
		return command;
	}

	public static UICommand createCommand(ValueBinding value,
			MethodBinding action) {
		final UICommand command = createCommand(action);
		command.setValueBinding("value", value);
		return command;
	}

	public static UICommand createCommand(String messageId, MethodBinding action) {
		final UICommand command = createCommand(action);
		command.setValueBinding("value", new Text(messageId));
		return command;
	}
}
